from vcCommand import *

app = getApplication()
global total_components, total_nodes, total_sets, total_triangles, total_lines, total_triangle_points, total_line_points, total_frames
global sharing_map

# Values at geometry which 
LARGE_SET_TRIANGLE_AMOUNT = 100000
LARGE_SET_TRIANGLE_POINT_AMOUNT = 100000

LARGE_SET_LINE_AMOUNT = 100000
LARGE_SET_LINE_POINT_AMOUNT = 100000

LARGE_NODE_SET_AMOUNT = 500

LARGE_NODE_TRIANGLE_AMOUNT = 100000
LARGE_NODE_TRIANGLE_POINT_AMOUNT = 100000

LARGE_NODE_LINE_AMOUNT = 100000
LARGE_NODE_LINE_POINT_AMOUNT = 100000



def ProcessNodes(node, comp):
    if node.Component != comp:
        return

    # Process this nodes behaviors
    ProcessNodeGeometry(node)

    # Process all child nodes if they belong to this component
    for child in node.Children:
        ProcessNodes(child, comp)


def ProcessNodeGeometry(node):
    global total_components, total_nodes, total_sets, total_triangles, total_lines, total_triangle_points, total_line_points, total_frames

    container = node.Geometry
    node_triangles = 0
    node_triangle_points = 0
    node_lines = 0
    node_line_points = 0
    node_frames = 0
  
    for set in container.GeometrySets:
        try:
            if set.Type == VC_TRIANGLESET:
                node_triangles += set.TriangleCount
                node_triangle_points += set.PointCount
                if set.TriangleCount > LARGE_SET_TRIANGLE_AMOUNT:
                    print "WARNING: ",node.Component.Name," set triangle amount exceeds recommenned limit (",LARGE_SET_TRIANGLE_AMOUNT,") ", set.TriangleCount
                if set.PointCount > LARGE_SET_TRIANGLE_POINT_AMOUNT:
                    print "WARNING: ",node.Component.Name," set triangle point amount exceeds recommenned limit (",LARGE_SET_TRIANGLE_POINT_AMOUNT,") ", set.PointCount

            if set.Type == VC_COMPACTLINESET:
                node_lines += set.LineCount
                points = 0
                for ii in range(0,set.LineCount):
                    points += set.getLinePointCount(ii)

                node_line_points += points
                if set.LineCount > LARGE_SET_LINE_AMOUNT:
                    print "WARNING: ",node.Component.Name," set line amount exceeds recommenned limit (",LARGE_SET_LINE_AMOUNT,") ", set.LineCount
                if points > LARGE_SET_LINE_POINT_AMOUNT:
                    print "WARNING: ",node.Component.Name," set line point amount exceeds recommenned limit (",LARGE_SET_LINE_POINT_AMOUNT,") ", points

            if set.Type == VC_FRAMESET:
                node_frames += set.FrameCount

        except:
            pass   


    if node_triangles > LARGE_NODE_TRIANGLE_AMOUNT:
        print "WARNING: ",node.Component.Name," node triangle amount exceeds recommenned limit (",LARGE_NODE_TRIANGLE_AMOUNT,") ", node_triangles
    if node_lines > LARGE_NODE_LINE_AMOUNT:
        print "WARNING: ",node.Component.Name," node triangle point amount exceeds recommenned limit (",LARGE_NODE_LINE_AMOUNT,") ", node_lines

    total_nodes += 1
    total_sets += container.GeometrySetCount

    total_triangles += node_triangles
    total_triangle_points += node_triangle_points

    total_lines += node_lines
    total_line_points += node_line_points

    total_frames += node_frames


def ProcessComponent(comp):
    global sharing_map

    # Update components sharing
    try:
        shared_nodes = sharing_map[comp.SharedFromNode]
        shared_nodes += 1
        sharing_map[comp.SharedFromNode] = shared_nodes
    except:
        sharing_map[comp.SharedFromNode] = 1

    # Analyze component geometry amounts
    ProcessNodes(comp, comp)


def process():
    global total_components, total_nodes, total_sets, total_triangles, total_lines, total_triangle_points, total_line_points, total_frames
    global sharing_map
    total_components = 0
    total_nodes = 0
    total_sets = 0
    total_triangles = 0
    total_lines = 0
    total_triangle_points = 0
    total_line_points = 0
    total_frames = 0
    sharing_map = {}

    print "     Analyze layout component geometry amounts and warn about excessive geometry"
    print "\n"
    print "WARNINGS..."
    comps = app.Components
    for comp in comps:
        # print "Component:",comp.Name  
        total_components += 1
        ProcessComponent(comp)
    
    print "\n"
    print "     LAYOUT HAS"
    print "Components:", total_components
    print "Nodes:", total_nodes
    print "Geometry Sets:", total_sets
    print "Triangles:", total_triangles, " Triangle points:", total_triangle_points
    if total_triangles > 0:
        if (float(total_triangle_points)/float(total_triangles)) > 2.0:
            print "WARNING: much larger point count in relation to triangle amount indicates usually non shared triangles which creates faceted shading, increases memory usage and slows down graphics"
    print "Lines:", total_lines, " Line points:", total_line_points
    print "Frames:", total_frames

    if total_components > 0:
        print "\n"
        print "     AVERAGE COMPONENT HAS"
        print "Geometry Sets:", float(total_sets)/float(total_components)
        print "Triangles:", float(total_triangles)/float(total_components)
        print "Triangle points:", float(total_triangle_points)/float(total_components)
        print "Lines:", float(total_lines)/float(total_components)
        print "Lines points:", float(total_line_points)/float(total_components)

        print "\n"
        print "     COMPONENT SHARING"
        print "Component is shared in average:", float(total_components)/float(len(sharing_map))

        print "\n"
        print "     SHARING BY COMPONENT (NAME)"
        for k, v in sharing_map.items():
           print "Component:",k.Name," shared:",v

addState( process )
