from vcCommand import *
import os

app = getApplication()
cmd = getCommand()

output_comp_path = os.path.join(os.path.dirname(cmd.getCommandPath()[8:]), "output_comp.csv")
DELIM = ","

def process():
  global totalCount, totalTime

  if app.Simulation.ProfilingEnabled == False:
      print "WARNING: Profiling is not enabled. Enable profiling and run simulation first!"
      return

  entries = []
  header = "Component" + DELIM + "Calls" + DELIM + "Time\r\n"
  totalCount = 0
  totalTime = 0.0
  for comp in app.Components:
    count, time = comp.getProfilingInfo()
    entries.append([comp.Name, count, time])
    totalCount += count
    totalTime += time

  entries.sort(key=lambda x: x[-1], reverse=True)
  write_to_file(output_comp_path, entries, header)
  print_top_most_time(entries, header)

  print_summary()


def write_to_file(output_path, sorted_entries, header):
  with open(output_path, "w") as f:
    output = header
    for entry in sorted_entries:
      output += DELIM.join((str(e) for e in entry))
      output += "\n"
    f.write(output)

def print_top_most_time(sorted_entries, header, limit=20):
  print "-"*50
  print "Top {} most time:".format(limit)
  for h in header.split(DELIM):
    print "{:<50}".format(str(h)),
  print "\n"
  print "_"*150

  for entry in sorted_entries[:limit]:
    output = ""
    for e in entry:
      print "{:<50}".format(str(e)),
    print "\n"

  print "\n.\n..\n"

def print_summary():
  print "Components\tTotalCalls\tTotalTime"
  print "_"*150
  print str(len(app.Components)) + "\t\t" + str(totalCount) + "\t\t" + str(totalTime)
  print "-"*50
  print "Full records in:"
  print output_comp_path
  print "-"*50


addState( process )
